94057c3d3e
PR Test (NPU) / check-changes (push) Has been cancelled
PR Test (NPU) / pr-gate (push) Has been cancelled
PR Test (NPU) / set-image-config (push) Has been cancelled
PR Test (NPU) / stage-b-test-1-npu-a2 (0) (push) Has been cancelled
PR Test (NPU) / stage-b-test-1-npu-a2 (1) (push) Has been cancelled
PR Test (NPU) / stage-b-test-2-npu-a2 (0) (push) Has been cancelled
PR Test (NPU) / stage-b-test-2-npu-a2 (1) (push) Has been cancelled
PR Test (NPU) / stage-b-test-4-npu-a3 (push) Has been cancelled
PR Test (NPU) / stage-b-test-16-npu-a3 (push) Has been cancelled
PR Test (NPU) / multimodal-gen-test-1-npu-a3 (push) Has been cancelled
PR Test (NPU) / multimodal-gen-test-2-npu-a3 (push) Has been cancelled
PR Test (Arm64) / pr-gate (push) Has been cancelled
PR Test (Arm64) / check-changes (push) Has been cancelled
PR Test (Arm64) / build-test (push) Has been cancelled
PR Test (sgl-router) / gate (push) Has been cancelled
PR Test (sgl-router) / tier-1 — lint (push) Has been cancelled
PR Test (sgl-router) / tier-2 — build + test (push) Has been cancelled
PR Test (sgl-router) / tier-3 — docker (placeholder) (push) Has been cancelled
PR Test (sgl-router) / tier-3 — k8s integration (push) Has been cancelled
PR Test (sgl-router) / tier-3 — e2e (push) Has been cancelled
PR Test (sgl-router) / finish (push) Has been cancelled
PR Test (NPU) / single-node-poc (map[name:qwen3_6_27b_w8a8_1p_in64k_out1k_50ms runner:linux-aarch64-a3-2 test_case:test/registered/ascend/performance/qwen3_6_27b/test_npu_qwen3_6_27b_w8a8_1p_in64k_out1k_50ms.py test_type:perf]) (push) Has been cancelled
PR Test (NPU) / pr-test-npu-finish (push) Has been cancelled
PR Test (Xeon) / pr-gate (push) Has been cancelled
PR Test (Xeon) / check-changes (push) Has been cancelled
PR Test (Xeon) / build-test (, xeon-gnr, base-b-test-cpu) (push) Has been cancelled
PR Test (XPU) / check-changes (push) Has been cancelled
PR Test (XPU) / pr-gate (push) Has been cancelled
PR Test (XPU) / stage-a-test-1-gpu-xpu (push) Has been cancelled
PR Test (XPU) / wait-for-stage-a (push) Has been cancelled
PR Test (XPU) / stage-b-test-1-gpu-xpu (push) Has been cancelled
PR Test (XPU) / finish (push) Has been cancelled
CI Model Inventory / build-inventory (push) Has been cancelled
Lint / lint (push) Has been cancelled
PR Benchmark (SMG Components) / Benchmark Compilation Check (push) Has been cancelled
PR Benchmark (SMG Components) / Benchmark - Manual Policy (push) Has been cancelled
PR Benchmark (SMG Components) / Benchmark - Request Processing (push) Has been cancelled
PR Benchmark (SMG Components) / Benchmark Summary (push) Has been cancelled
PR Test (SMG) / build-wheel (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on windows (x86_64 - auto) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on macos (x86_64 - auto) (push) Has been cancelled
PR Test (SMG) / python-unit-tests (push) Has been cancelled
PR Test (SMG) / unit-tests (push) Has been cancelled
PR Test (SMG) / benchmarks (push) Has been cancelled
PR Test (SMG) / chat-completions (push) Has been cancelled
PR Test (SMG) / chat-completions-4gpu (push) Has been cancelled
PR Test (SMG) / e2e (push) Has been cancelled
PR Test (SMG) / docker-build-test (push) Has been cancelled
PR Test (SMG) / k8s-integration (push) Has been cancelled
PR Test (SMG) / finish (push) Has been cancelled
PR Test (SMG) / summarize-benchmarks (push) Has been cancelled
Release SGLang Model Gateway Docker Image / publish (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on macos (aarch64 - auto) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on linux (aarch64 - auto) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on linux (x86_64 - auto) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on linux (aarch64 - musllinux_1_1) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on linux (x86_64 - musllinux_1_1) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / Build SDist (push) Has been cancelled
Release SGLang Model Gateway to PyPI / Upload to PyPI (push) Has been cancelled
Release SGLang Kernels / build-cu129-matrix (aarch64, 12.9, 3.10, arm-kernel-build-node) (push) Has been cancelled
Release SGLang Kernels / build-cu129-matrix (x86_64, 12.9, 3.10, x64-kernel-build-node) (push) Has been cancelled
Release SGLang Kernels / release-cu129 (push) Has been cancelled
Release SGLang Kernels / build-cu130-matrix (aarch64, 13.0, 3.10, arm-kernel-build-node) (push) Has been cancelled
Release SGLang Kernels / build-cu130-matrix (x86_64, 13.0, 3.10, x64-kernel-build-node) (push) Has been cancelled
Release SGLang Kernels / release-cu130 (push) Has been cancelled
Release SGLang Kernels / build-rocm-matrix (3.10, 700) (push) Has been cancelled
Release SGLang Kernels / build-rocm-matrix (3.10, 720) (push) Has been cancelled
Release SGLang Kernels / release-rocm700 (push) Has been cancelled
Release SGLang Kernels / release-rocm720 (push) Has been cancelled
Release SGLang Kernels / build-musa43 (43, 3.10) (push) Has been cancelled
Release SGLang Kernels / release-musa43 (push) Has been cancelled
123 lines
3.0 KiB
Python
123 lines
3.0 KiB
Python
from __future__ import annotations
|
|
|
|
import torch
|
|
import triton # type: ignore
|
|
import triton.language as tl # type: ignore
|
|
|
|
|
|
@triton.jit
|
|
def _fused_cat_pad_5d_kernel(
|
|
x_ptr,
|
|
cache_ptr,
|
|
out_ptr,
|
|
total,
|
|
channels,
|
|
t_size,
|
|
h_size,
|
|
w_size,
|
|
cache_t,
|
|
out_t,
|
|
out_h,
|
|
out_w,
|
|
pad_d_left,
|
|
pad_h_top,
|
|
pad_w_left,
|
|
block_size: tl.constexpr,
|
|
):
|
|
offsets = tl.program_id(0) * block_size + tl.arange(0, block_size)
|
|
mask = offsets < total
|
|
|
|
ow = offsets % out_w
|
|
tmp = offsets // out_w
|
|
oh = tmp % out_h
|
|
tmp = tmp // out_h
|
|
out = tmp % out_t
|
|
tmp = tmp // out_t
|
|
oc = tmp % channels
|
|
ob = tmp // channels
|
|
|
|
iw = ow - pad_w_left
|
|
ih = oh - pad_h_top
|
|
src_t = out - pad_d_left
|
|
|
|
valid = (
|
|
mask
|
|
& (iw >= 0)
|
|
& (iw < w_size)
|
|
& (ih >= 0)
|
|
& (ih < h_size)
|
|
& (src_t >= 0)
|
|
& (src_t < cache_t + t_size)
|
|
)
|
|
from_cache = src_t < cache_t
|
|
|
|
x_t = src_t - cache_t
|
|
clamped_iw = tl.minimum(tl.maximum(iw, 0), w_size - 1)
|
|
clamped_ih = tl.minimum(tl.maximum(ih, 0), h_size - 1)
|
|
clamped_x_t = tl.minimum(tl.maximum(x_t, 0), t_size - 1)
|
|
clamped_src_t = tl.minimum(tl.maximum(src_t, 0), cache_t - 1)
|
|
|
|
x_offsets = (
|
|
((ob * channels + oc) * t_size + clamped_x_t) * h_size + clamped_ih
|
|
) * w_size + clamped_iw
|
|
cache_offsets = (
|
|
((ob * channels + oc) * cache_t + clamped_src_t) * h_size + clamped_ih
|
|
) * w_size + clamped_iw
|
|
|
|
x_vals = tl.load(x_ptr + x_offsets, mask=valid & ~from_cache, other=0.0)
|
|
cache_vals = tl.load(cache_ptr + cache_offsets, mask=valid & from_cache, other=0.0)
|
|
vals = tl.where(from_cache, cache_vals, x_vals)
|
|
tl.store(out_ptr + offsets, vals, mask=mask)
|
|
|
|
|
|
def fused_causal_conv3d_cat_pad(
|
|
x: torch.Tensor,
|
|
cache_x: torch.Tensor,
|
|
padding: list[int] | tuple[int, ...],
|
|
) -> torch.Tensor:
|
|
width_left, width_right, height_top, height_bottom, depth_left, depth_right = (
|
|
padding
|
|
)
|
|
depth_left -= cache_x.shape[2]
|
|
assert depth_left >= 0
|
|
assert depth_right == 0
|
|
assert width_left == width_right
|
|
assert height_top == height_bottom
|
|
|
|
bsz, channels, t_size, h_size, w_size = x.shape
|
|
cache_t = cache_x.shape[2]
|
|
out = torch.empty(
|
|
(
|
|
bsz,
|
|
channels,
|
|
t_size + cache_t + depth_left + depth_right,
|
|
h_size + height_top + height_bottom,
|
|
w_size + width_left + width_right,
|
|
),
|
|
device=x.device,
|
|
dtype=x.dtype,
|
|
)
|
|
block_size = 256
|
|
total = out.numel()
|
|
grid = (triton.cdiv(total, block_size),)
|
|
with torch.get_device_module().device(x.device):
|
|
_fused_cat_pad_5d_kernel[grid](
|
|
x,
|
|
cache_x,
|
|
out,
|
|
total,
|
|
channels,
|
|
t_size,
|
|
h_size,
|
|
w_size,
|
|
cache_t,
|
|
out.shape[2],
|
|
out.shape[3],
|
|
out.shape[4],
|
|
depth_left,
|
|
height_top,
|
|
width_left,
|
|
block_size,
|
|
)
|
|
return out
|